<?php
/**
 *  Class to display an analog clock.
 *
 *  Requirement: 
 *    - PHP 5.2.x or above
 *    - GD  2.0.1 or above  
 *  
 *  Some of the highlights:
 *
 *      -   Autoscale depending on width (min 30px) 
 *      -   set quadrant color background
 *      -   set quadrant color foreground
 *      -   set image color background
 *      -   set color of seconds hand  
 *      -   set color of minutes hand
 *      -   set color of hours hand              
 *      -   set autotime (server-side mode)
 *      -   Choose to display or not seconds
 *
 *  This work is licensed under the Creative Commons Attribution-NonCommercial-NoDerivs 2.5 License.
 *  To view a copy of this license, visit {@link http://creativecommons.org/licenses/by/2.5/it/} 
 *  For more resources visit {@link http://www.mariospada.net}
 *
 *  @name       Analogclock
 *  @package    Analogclock
 *  @version    1.0.0 alfa (last revision: July 06, 2008)
 *  @author     Mario Spada <mario@mariospada.net>
 *  @copyright  (c) 2008 Mario Spada
 *  @example    test.php
 */
class Analogclock 
{
  public $clockWidth;
  protected $img;
  protected $shiftAngle;
  protected $r;
  protected $colorBackgr;
  protected $imgBackgr;
  protected $colorForegr;
  protected $colorText;
  protected $colorMin;
  protected $colorHour;
  protected $colorSec;
  protected $displayText = true;
  protected $textSize = 5;
  protected $autotime = false;
  protected $showSeconds = true;
  
  public function __construct($clockWidth=200)
  {
    $this->clockWidth = $clockWidth;
    if ($this->clockWidth < 30 ) {
      $this->clockWidth = 30;
    }
    if ($this->clockWidth < 90 ) {
      $this->displayText = false;
    }
    if ($this->clockWidth >= 90 && $this->clockWidth < 150) {
      $this->textSize = 4;
    }

    $this->img = @imagecreatetruecolor($clockWidth, $clockWidth)
            or die("Cannot Initialize new GD image stream"); 
    $this->shiftAngle = M_PI + M_PI / 2 ;
    $this->r = ($this->clockWidth) / 2;

    $this->colorBackgr = imageColorAllocate($this->img, 255, 255, 255);
    $this->imgBackgr = imageColorAllocate($this->img, 255, 255, 255);
    $this->colorForegr = imageColorAllocate($this->img, 0, 0, 0);
    $this->colorText   = imageColorAllocate($this->img, 0, 255, 0);
    $this->colorMin    = imageColorAllocate($this->img, 0, 0, 0);
    $this->colorHour   = imageColorAllocate($this->img, 0, 0, 0);
    $this->colorSec    = imageColorAllocate($this->img, 255, 0, 0);
  }

  public function setColorBackground($red, $green, $blue)
  {
    $this->colorBackgr = imageColorAllocate($this->img, $red, $green, $blue);
  }
  public function setImgBackground($red, $green, $blue)
  {
    $this->imgBackgr = imageColorAllocate($this->img, $red, $green, $blue);
  }
  public function setColorForeground($red, $green, $blue)
  {
    $this->colorForegr = imageColorAllocate($this->img, $red, $green, $blue);
  }
  public function setColorMin($red, $green, $blue)
  {
    $this->colorMin = imageColorAllocate($this->img, $red, $green, $blue);
  }
  public function setColorHour($red, $green, $blue)
  {
    $this->colorHour = imageColorAllocate($this->img, $red, $green, $blue);
  }
  public function setColorSec($red, $green, $blue)
  {
    $this->colorHour = imageColorAllocate($this->img, $red, $green, $blue);
  }
  public function setAutotime($value)
  {
    $this->autotime = $value;
  }
  public function showSeconds($value)
  {
    $this->showSeconds = $value;
  }
  
  private function drawBackground()
  {
    imagefill($this->img, 0, 0, $this->imgBackgr);
    imageantialias($this->img, true );

    imagefilledellipse  ($this->img, $this->clockWidth/2, $this->clockWidth/2, 
                         $this->r, $this->r, $this->colorBackgr );    
  }

  private function imagelinethick($x1, $y1, $x2, $y2, $color, $thick = 1)
  {
    if ($thick == 1) {
        return imageline($this->img, $x1, $y1, $x2, $y2, $color);
    }
    $t = $thick / 2 - 0.5;
    if ($x1 == $x2 || $y1 == $y2) {
        return imagefilledrectangle($this->img, round(min($x1, $x2) - $t), 
                                            round(min($y1, $y2) - $t), 
                                            round(max($x1, $x2) + $t), 
                                            round(max($y1, $y2) + $t), 
                                            $color);
    }
    $k = ($y2 - $y1) / ($x2 - $x1); //y = kx + q
    $a = $t / sqrt(1 + pow($k, 2));
    $points = array(
        round($x1 - (1+$k)*$a), round($y1 + (1-$k)*$a),
        round($x1 - (1-$k)*$a), round($y1 - (1+$k)*$a),
        round($x2 + (1+$k)*$a), round($y2 - (1-$k)*$a),
        round($x2 + (1-$k)*$a), round($y2 + (1+$k)*$a),
    );
    imagefilledpolygon($this->img, $points, 4, $color);
    return imagepolygon($this->img, $points, 4, $color);
  }

  private function drawQuadrant()
  {
    $gamma = $this->shiftAngle ;
    
    for ($i = 1; $i <= 12; $i++)
    {
      $gamma += ((M_PI * 1 / 6) * $i) ;
      $X = $this->r + (cos($gamma) * $this->r * 0.9);
      $Y = $this->r + (sin($gamma) * $this->r * 0.9);
      $X1 = $this->r + (cos($gamma) * $this->r);
      $Y1 = $this->r + (sin($gamma) * $this->r);
      $thickness = ($i == 3 || $i == 6 || $i == 9 || $i == 12) ? 4 : 1; 
      $this->imagelinethick($X, $Y, $X1, $Y1, $this->colorForegr, $thickness);
      $gamma = $this->shiftAngle ;
    }
  }

  public function display($hour=0, $minutes=0, $seconds=0)
  {
    $this->drawBackground();
    $this->drawQuadrant();
    
    if($this->autotime)
    {
      $today = getdate();
      $minutes = $today['minutes'];
      $hour    = $today['hours'];
      $seconds = $today['seconds']; 
    }
    
    if ($this->showSeconds)
    {
      $timeText = sprintf("%02d:%02d:%02d",$hour,$minutes,$seconds);
      $xImgShift = 30;
    }
    else
    {
      $timeText = sprintf("%02d:%02d",$hour,$minutes);
      $xImgShift = 20;
    }
   
    if ($this->displayText)
    {
      imageString($this->img, $this->textSize, 
                  $this->r-$xImgShift, $this->r+10, $timeText, $this->colorText);
    }
    
    $alfa = $beta = $delta = $this->shiftAngle ;
    
    $delta += (M_PI * 1 / 30) * $seconds ;
    $alfa += (M_PI * 1 / 30) * $minutes ;
    $beta += (M_PI * 1 / 6) * ($hour + $minutes/60) ;
    // seconds
    $newX0 = $this->r + cos($delta) * ($this->r - 5);
    $newY0 = $this->r + sin($delta) * ($this->r - 5) ;
    // minutes
    $newX1 = $this->r + cos($alfa) * ($this->r * 3 / 4) ;
    $newY1 = $this->r + sin($alfa) * ($this->r * 3 / 4) ;
    // hours
    $newX2 = $this->r + cos($beta) * ($this->r *  2 / 3) ;
    $newY2 = $this->r + sin($beta) * ($this->r * 2 / 3) ;
    
    if ($this->showSeconds)
    {
      imageLine($this->img, $this->r, $this->r, $newX0, $newY0, $this->colorSec);
    }
    $this->imagelinethick($this->r, $this->r, $newX1, $newY1, $this->colorMin, 2);
    $this->imagelinethick($this->r, $this->r, $newX2, $newY2, $this->colorHour, 3);
                               
    // set the content type
    //header("Content-type:  image/gif");
    header("Content-type:  image/png");
    // create an interlaced image for better loading in the browser
    imageInterlace($this->img, 1);
    // now send the picture to the client (this outputs all image data directly)
    imagePNG($this->img);
  }
}
?>
